#!/usr/bin/env node
// cli.js
import { Command } from 'commander';
import { runTextify } from './lib/main.js';
import { applyTextifiedCodebase } from './lib/apply.js'; // Import the new function
import { createRequire } from 'module';
import path from 'path'; // Needed for resolving paths

const require = createRequire(import.meta.url);
const pkg = require('./package.json'); // Load package.json for version

const program = new Command();

// --- Textify (Generate) Command ---
program
  .name('textify-codebase') // Set the base command name
  .version(pkg.version)
  .description(pkg.description + ' Use "textify-codebase apply --help" for the apply command.')
  .argument('[directory]', 'The directory to process', '.') // Default to current directory
  .option('-o, --output <file>', 'Specify output file path (default: stdout)')
  .option('-i, --ignore <pattern...>', 'Add custom ignore patterns (repeatable)')
  .option('-d, --max-depth <depth>', 'Maximum depth for directory tree', (value) => {
    const parsed = parseInt(value, 10);
    if (isNaN(parsed) || parsed < 1) {
      throw new program.InvalidArgumentError('Max depth must be a positive integer.');
    }
    return parsed;
  }, 5)
  .option('--no-gitignore', 'Disable reading .gitignore files')
  .option('--no-default-ignore', 'Disable built-in default ignore patterns')
  .option('--scrub', 'Enable basic secret scrubbing (use with caution!)')
  .option('--encoding <encoding>', 'Specify file reading encoding', 'utf-8')
  .option('--tree-only', 'Only output the directory tree structure')
  .option('--content-only', 'Only output the file contents (no tree)')
  .action(async (directory, options) => {
    const textifyOptions = {
      directory: path.resolve(directory), // Resolve directory path early
      output: options.output ? path.resolve(options.output) : undefined, // Resolve output path early
      ignorePatterns: options.ignore || [],
      maxDepth: options.maxDepth,
      useGitignore: options.gitignore, // Commander handles --no-X as false
      useDefaultIgnore: options.defaultIgnore, // Commander handles --no-X as false
      scrub: options.scrub || false,
      encoding: options.encoding,
      treeOnly: options.treeOnly || false,
      contentOnly: options.contentOnly || false,
    };

    if (textifyOptions.treeOnly && textifyOptions.contentOnly) {
      console.error("Error: Cannot use --tree-only and --content-only together.");
      process.exit(1);
    }

    try {
      await runTextify(textifyOptions);
    } catch (error) {
      console.error("\nAn unexpected error occurred during textify:", error);
      process.exit(1);
    }
  });

// --- Apply Command ---
program
  .command('apply')
  .description('Apply a textified codebase file back to the file system (OVERWRITES FILES!)')
  .argument('<inputFile>', 'The textified codebase file to apply (e.g., codebase.txt)')
  .option('-t, --target <directory>', 'The target directory to apply changes to', '.')
  .option('-y, --yes', 'Skip confirmation prompt (use with extreme caution!)')
  .option('--dry-run', 'Show what changes would be made without writing files')
  .option('--encoding <encoding>', 'Specify file writing encoding', 'utf-8')
  .action(async (inputFile, options) => {
    const applyOptions = {
      inputFile: path.resolve(inputFile), // Resolve input path
      targetDirectory: path.resolve(options.target), // Resolve target dir path
      skipConfirmation: options.yes || false,
      dryRun: options.dryRun || false,
      encoding: options.encoding,
    };

    try {
      await applyTextifiedCodebase(applyOptions);
    } catch (error) {
      console.error("\nAn unexpected error occurred during apply:", error);
      process.exit(1);
    }
  });


// Make sure to parse arguments for both commands
program.parse(process.argv);

// Handle cases where no command is given or is ambiguous
// Commander's default behavior handles the main command well.
// We only need to explicitly check if 'apply' was intended but lacked arguments.
if (process.argv.length > 2 && process.argv[2] === 'apply' && process.argv.length < 4) {
  // 'apply' command called without input file argument
  program.commands.find(cmd => cmd.name() === 'apply').outputHelp();
  process.exit(1);
}
// Note: If just 'textify-codebase' is run, the default action with '.' directory is executed.
// If 'textify-codebase some-dir' is run, the action is executed with 'some-dir'.
// If 'textify-codebase apply file.txt' is run, the apply action is executed.