// lib/apply.js
import fs from 'fs/promises';
import path from 'path';
import readline from 'readline';
import chalk from 'chalk';

// Regex to find file blocks, allowing for variations in whitespace and paths
const fileMarkerRegex = /^--- (START|END) FILE: (.*?) --- *$/; // Added optional trailing spaces *

async function promptUser(query) {
    const rl = readline.createInterface({
        input: process.stdin,
        output: process.stdout,
    });

    return new Promise(resolve => rl.question(query, ans => {
        rl.close();
        resolve(ans.trim().toLowerCase());
    }));
}

export async function applyTextifiedCodebase(options) {
    const { inputFile, targetDirectory, skipConfirmation, dryRun, encoding } = options;

    console.log(chalk.blue(`Applying textified codebase from: ${inputFile}`));
    console.log(chalk.blue(`Target directory: ${targetDirectory}`));
    if (dryRun) {
        console.log(chalk.yellow.bold('--- DRY RUN MODE ---'));
        console.log(chalk.yellow('No files will actually be written.'));
    }

    // --- Read and Parse Input File ---
    let content;
    try {
        content = await fs.readFile(inputFile, { encoding: encoding });
    } catch (err) {
        console.error(chalk.red(`Error reading input file ${inputFile}: ${err.message}`));
        process.exit(1);
    }

    const lines = content.split(/\r?\n/); // Handle both LF and CRLF line endings
    const filesToWrite = [];
    let currentFileRelativePath = null;
    let currentContentLines = [];
    let lineNum = 0;

    for (const line of lines) {
        lineNum++;
        const match = line.match(fileMarkerRegex);
        if (match) {
            const type = match[1]; // START or END
            // Normalize path separators in the captured path to POSIX for internal consistency
            const relativePath = match[2].trim().split(path.win32.sep).join(path.posix.sep);

            if (type === 'START') {
                if (currentFileRelativePath) {
                    console.warn(chalk.yellow(`Warning (Line ${lineNum}): Found nested START marker for "${relativePath}" inside "${currentFileRelativePath}". Ending previous file implicitly.`));
                    // Implicitly end the previous file
                    filesToWrite.push({
                        relativePath: currentFileRelativePath, // Use POSIX path internally
                        content: currentContentLines.join('\n'),
                        sourceLine: lineNum // For reference
                    });
                    currentContentLines = []; // Reset content
                }
                currentFileRelativePath = relativePath;
                currentContentLines = []; // Reset content for the new file
            } else if (type === 'END') {
                if (!currentFileRelativePath) {
                    console.warn(chalk.yellow(`Warning (Line ${lineNum}): Found END marker for "${relativePath}" without a matching START marker. Ignoring.`));
                } else if (currentFileRelativePath !== relativePath) {
                    console.warn(chalk.yellow(`Warning (Line ${lineNum}): Found END marker for "${relativePath}" but expected END marker for "${currentFileRelativePath}". Assuming end of "${currentFileRelativePath}".`));
                    // Process the file we thought we were in
                    filesToWrite.push({
                        relativePath: currentFileRelativePath, // Use POSIX path
                        content: currentContentLines.join('\n'),
                        sourceLine: lineNum // For reference
                    });
                    currentFileRelativePath = null; // Reset state
                    currentContentLines = [];
                } else {
                    // Correct END marker found
                    filesToWrite.push({
                        relativePath: currentFileRelativePath, // Use POSIX path
                        content: currentContentLines.join('\n'),
                        sourceLine: lineNum // For reference
                    });
                    currentFileRelativePath = null; // Reset state
                    currentContentLines = [];
                }
            }
        } else if (currentFileRelativePath !== null) {
            // Line is part of the current file's content
            currentContentLines.push(line);
        }
        // Ignore lines outside of START/END blocks (like the tree, separators, blank lines between files)
    }

    if (currentFileRelativePath) {
        console.warn(chalk.yellow(`Warning: Input file ended while still processing "${currentFileRelativePath}" (started near line ${filesToWrite[filesToWrite.length - 1]?.sourceLine || 1}). File content might be incomplete or the END marker is missing.`));
        // Decide whether to add the potentially incomplete file. Let's add it with a warning.
        filesToWrite.push({
            relativePath: currentFileRelativePath,
            content: currentContentLines.join('\n'),
            isIncomplete: true,
            sourceLine: lineNum
        });
    }

    if (filesToWrite.length === 0) {
        console.log(chalk.yellow('No valid file blocks (--- START FILE: ... --- ... --- END FILE: ... ---) found in the input file. Nothing to apply.'));
        return;
    }

    console.log(chalk.blue(`\nFound ${filesToWrite.length} files to potentially write:`));
    filesToWrite.forEach(f => {
        console.log(chalk.cyan(`  - ${f.relativePath}${f.isIncomplete ? ' (Warning: Potentially incomplete)' : ''}`));
    });

    // --- Confirmation ---
    if (!skipConfirmation && !dryRun) {
        console.log(chalk.red.bold('\nWARNING: This operation will OVERWRITE existing files in the target directory!'));
        const answer = await promptUser(`Are you sure you want to apply these changes to "${targetDirectory}"? (y/N) `);
        if (answer !== 'y') {
            console.log(chalk.yellow('Operation cancelled by user.'));
            process.exit(0);
        }
    } else if (!dryRun && skipConfirmation) {
        console.log(chalk.yellow('\nSkipping confirmation prompt due to --yes flag. Proceeding with file writing...'));
    }


    // --- Write Files ---
    console.log(chalk.blue('\nApplying changes...'));
    let writeCount = 0;
    let errorCount = 0;

    for (const file of filesToWrite) {
        // IMPORTANT: Construct the absolute target path
        // Use path.resolve to handle the target directory and the relative path robustly.
        // The relative path from the file block should be joined to the target directory.
        // Ensure the relative path doesn't try to escape the target directory.
        const targetPath = path.resolve(targetDirectory, file.relativePath);

        // Security Check: Ensure the resolved path is still within the target directory.
        if (!targetPath.startsWith(path.resolve(targetDirectory) + path.sep) && targetPath !== path.resolve(targetDirectory)) {
            // This check prevents paths like ../../etc/passwd resolving outside the target dir.
            // It checks if the resolved absolute path starts with the resolved absolute target directory path.
            console.error(chalk.red(`  Error: Path "${file.relativePath}" resolves outside the target directory "${targetDirectory}". Skipping.`));
            errorCount++;
            continue;
        }


        console.log(`  ${dryRun ? '[Dry Run] Would write' : 'Writing'}: ${path.relative(process.cwd(), targetPath)} ${file.isIncomplete ? chalk.yellow('(Potentially Incomplete)') : ''}`);

        if (!dryRun) {
            try {
                // Ensure the directory for the file exists
                const dirName = path.dirname(targetPath);
                // Check if directory needs to be created (it might already exist)
                // Use stat to check existence first to avoid unnecessary mkdir calls/errors?
                // fs.mkdir with recursive: true handles existing directories gracefully.
                await fs.mkdir(dirName, { recursive: true });

                // Write the file, overwriting if it exists
                await fs.writeFile(targetPath, file.content, { encoding: encoding });
                writeCount++;
            } catch (err) {
                console.error(chalk.red(`  Error writing file ${path.relative(process.cwd(), targetPath)}: ${err.message}`));
                errorCount++;
            }
        } else {
            // In dry run, simulate success if path seems valid (passed the check above)
            writeCount++;
        }
    }

    // --- Final Summary ---
    console.log(chalk.green('\n--- Apply Operation Complete ---'));
    if (dryRun) {
        console.log(chalk.yellow.bold(`[Dry Run] Would have attempted to write ${writeCount} files.`));
        if (errorCount > 0) {
            console.log(chalk.yellow(`[Dry Run] Encountered ${errorCount} path errors (skipped).`));
        }
    } else {
        console.log(chalk.green(`Successfully wrote ${writeCount} files.`));
        if (errorCount > 0) {
            console.log(chalk.red(`Encountered ${errorCount} errors during writing or path validation.`));
        }
    }
    console.log(`Target directory: ${targetDirectory}`);
}