// lib/main.js
import fs from 'fs/promises';
import fssync from 'fs'; // For createWriteStream
import path from 'path';
import ignore from 'ignore';
import chalk from 'chalk';
import { generateTree } from './tree.js';
import { scrubContent, logScrubbingDisclaimer } from './scrub.js';

const DEFAULT_IGNORES = [
    // Core ignores
    'node_modules',
    '.git',
    // Build artifacts
    'dist',
    'build',
    'out',
    'coverage',
    '.next',
    '.nuxt',
    '.svelte-kit',
    // Logs & Lock files
    '*.log',
    'npm-debug.log*',
    'yarn-debug.log*',
    'yarn-error.log*',
    'pnpm-debug.log*',
    'package-lock.json',
    'yarn.lock',
    'pnpm-lock.yaml',
    // Environment & Secrets (KEEP THESE!)
    '.env',
    '.env.*',
    '!.env.example',
    '!.env.sample',
    'secrets.yml',
    '*.pem',
    '*.key',
    // OS specific
    '.DS_Store',
    'Thumbs.db',
    // Common binary file extensions
    '*.png', '*.jpg', '*.jpeg', '*.gif', '*.bmp', '*.tiff', '*.ico', '*.webp',
    '*.pdf', '*.doc', '*.docx', '*.xls', '*.xlsx', '*.ppt', '*.pptx',
    '*.zip', '*.tar', '*.gz', '*.rar', '*.7z', '*.tgz', '*.bz2',
    '*.mp3', '*.wav', '*.ogg', '*.aac', '*.flac',
    '*.mp4', '*.avi', '*.mov', '*.wmv', '*.mkv', '*.webm',
    '*.exe', '*.dll', '*.so', '*.dylib', '*.app', '*.dmg', '*.pkg',
    '*.o', '*.a', '*.obj', '*.lib', '*.wasm',
    '*.pyc', '*.pyo', '*.pyd',
    '*.class', '*.jar',
    '*.sqlite', '*.sqlite3', '*.db', '*.db3', '*.sqlitedb',
    '*.woff', '*.woff2', '*.eot', '*.ttf', '*.otf',
    // IDE/Editor specific
    '.vscode',
    '.idea',
    '*.swp',
    '*.swo',
];

async function findGitIgnore(startDir) {
    let currentDir = path.resolve(startDir);
    const root = path.parse(currentDir).root;
    while (currentDir !== root) {
        const gitignorePath = path.join(currentDir, '.gitignore');
        try {
            await fs.access(gitignorePath);
            return gitignorePath;
        } catch {
            // Not found, go up
        }
        const parentDir = path.dirname(currentDir);
        if (parentDir === currentDir) break; // Safety break
        currentDir = parentDir;
    }
    return null; // Not found
}

async function* walkDir(dir, ignoreInstance, rootDir, currentDepth = 1) {
    let entries;
    try {
        entries = await fs.readdir(dir, { withFileTypes: true });
    } catch (err) {
        if (err.code === 'EACCES' || err.code === 'EPERM') {
            console.warn(chalk.yellow(`   Skipping inaccessible directory: ${path.relative(rootDir, dir)}`));
        } else {
            console.warn(chalk.yellow(`   Warning: Could not read directory ${path.relative(rootDir, dir)}: ${err.message}`));
        }
        return; // Stop descent
    }

    for (const entry of entries) {
        const entryPath = path.join(dir, entry.name);
        // Use POSIX separators for relative path matching consistently
        const relativePath = path.relative(rootDir, entryPath).split(path.sep).join(path.posix.sep);
        const relativePathWithSlash = entry.isDirectory() ? relativePath + '/' : relativePath;

        // Test against ignore rules
        // Check both relative path and simple name (e.g. for node_modules at root)
        if (ignoreInstance.ignores(relativePathWithSlash) || ignoreInstance.ignores(entry.name)) {
            // console.log(`Ignoring: ${relativePathWithSlash} (Matched by rule)`); // Debug log
            continue;
        }

        if (entry.isDirectory()) {
            yield* walkDir(entryPath, ignoreInstance, rootDir, currentDepth + 1);
        } else if (entry.isFile()) {
            yield { path: entryPath, relativePath: relativePath }; // relativePath is already POSIX
        }
    }
}


export async function runTextify(options) {
    // Destructure options *including the resolved paths* from cli.js
    const { directory: rootDir, output: outputFilePath, ignorePatterns, maxDepth, useGitignore, useDefaultIgnore, scrub, encoding, treeOnly, contentOnly } = options;
    // rootDir and outputFilePath are already absolute paths here (if outputFilePath is defined)
    let outputStream;

    // --- Validate Root Directory ---
    try {
        const stats = await fs.stat(rootDir);
        if (!stats.isDirectory()) {
            console.error(chalk.red(`Error: The specified path "${rootDir}" is not a directory.`));
            process.exit(1);
        }
    } catch (err) {
        if (err.code === 'ENOENT') {
            console.error(chalk.red(`Error: The specified directory "${rootDir}" does not exist.`));
        } else {
            console.error(chalk.red(`Error: Could not access directory "${rootDir}": ${err.message}`));
        }
        process.exit(1);
    }

    // --- Initialize Ignore Instance ---
    const ig = ignore();
    if (useDefaultIgnore) {
        console.log(chalk.blue('Adding default ignore patterns...'));
        ig.add(DEFAULT_IGNORES);
    }

    if (useGitignore) {
        const gitignorePath = await findGitIgnore(rootDir);
        if (gitignorePath) {
            try {
                const gitignoreContent = await fs.readFile(gitignorePath, 'utf-8');
                ig.add(gitignoreContent);
                console.log(chalk.blue(`Loaded rules from ${path.relative(process.cwd(), gitignorePath)}`));
            } catch (err) {
                console.warn(chalk.yellow(`   Warning: Could not read ${path.relative(process.cwd(), gitignorePath)}: ${err.message}`));
            }
        } else {
            console.log(chalk.gray('   No .gitignore file found in project hierarchy.'));
        }
    }

    if (ignorePatterns && ignorePatterns.length > 0) {
        console.log(chalk.blue('Adding custom ignore patterns:'), ignorePatterns);
        ig.add(ignorePatterns);
    }

    // --- !! Add Output File to Ignore List !! ---
    if (outputFilePath) {
        // Calculate path relative to the root directory being scanned, using POSIX separators
        const relativeOutputPath = path.relative(rootDir, outputFilePath).split(path.sep).join(path.posix.sep);
        // Only add if it's within or equal to the root directory (not ../)
        // Use path.isAbsolute and check if it starts with rootDir to be safer?
        // path.relative handles this: if outputFilePath is outside rootDir, it will start with '..'
        if (!relativeOutputPath.startsWith('..') && relativeOutputPath !== '.') {
            console.log(chalk.blue(`Adding output file to ignore list: /${relativeOutputPath}`)); // Add leading slash for clarity
            // Add pattern relative to the root being scanned
            // ignore() usually works with patterns relative to the base where .gitignore is found (or cwd if none).
            // Since we're applying it from rootDir, the relative path should work directly.
            // Adding '/' prefix might make it anchor to the rootDir explicitly in ignore's logic.
            ig.add('/' + relativeOutputPath);
        } else if (relativeOutputPath === '.') {
            console.log(chalk.blue(`Adding output file (same as root dir?) to ignore list: /${path.basename(outputFilePath)}`));
            ig.add('/' + path.basename(outputFilePath)); // Ignore the file name if it's in the root
        } else {
            console.log(chalk.gray(`Output file ${outputFilePath} is outside the target directory ${rootDir}, no need to explicitly ignore.`));
        }
    }
    // --- End Output File Ignore ---


    // --- Prepare Output Stream ---
    if (outputFilePath) {
        console.log(chalk.blue(`Output will be written to: ${outputFilePath}`));
        try {
            // Ensure directory exists before creating stream
            await fs.mkdir(path.dirname(outputFilePath), { recursive: true });
            outputStream = fssync.createWriteStream(outputFilePath, { encoding: encoding });
            outputStream.on('error', (err) => {
                console.error(chalk.red(`Error writing to output file ${outputFilePath}: ${err.message}`));
                // Try to close stream? Might error again.
                process.exit(1);
            });
        } catch (err) {
            console.error(chalk.red(`Error preparing output file ${outputFilePath}: ${err.message}`));
            process.exit(1);
        }
    } else {
        console.log(chalk.blue('Output will be written to stdout.'));
        outputStream = process.stdout;
        // Prevent process.stdout from being closed by stream.end()
        outputStream.end = (cb) => { if (cb) cb(); }; // No-op end for stdout
    }

    const writeToOutput = (data) => {
        return new Promise((resolve, reject) => {
            // Add newline consistently here instead of expecting it in data
            const buffer = Buffer.from(data + '\n', encoding);
            const flushed = outputStream.write(buffer, (err) => {
                if (err) return reject(err);
                // If write is synchronous (flushed=true) or it's stdout, resolve via callback
                if (outputStream === process.stdout || flushed) {
                    resolve();
                }
                // If !flushed and not stdout, the 'drain' event will resolve it
            });

            if (!flushed && outputStream !== process.stdout) {
                // console.log('Write buffered, waiting for drain...'); // Debug log
                outputStream.once('drain', resolve);
            } else if (flushed && outputStream !== process.stdout) {
                // Already resolved via the callback for non-stdout when flushed
            } else if (outputStream === process.stdout) {
                // Already resolved via the callback for stdout
            }
        });
    };


    // --- Generate Directory Tree ---
    if (!contentOnly) {
        const treeString = await generateTree(rootDir, ig, maxDepth); // Pass ig instance
        await writeToOutput(treeString);
        await writeToOutput("\n" + "=".repeat(80)); // Removed extra \n here, writeToOutput adds it
    }

    if (treeOnly) {
        console.log(chalk.green('Tree only mode enabled. Skipping file content.'));
        if (outputStream !== process.stdout) {
            // Ensure stream is closed properly when finishing early
            await new Promise((resolve, reject) => {
                outputStream.end((err) => {
                    if (err) return reject(err);
                    resolve();
                });
            });
        }
        console.log(chalk.green('Directory tree generation complete.'));
        return; // Exit early
    }

    // --- Process Files ---
    if (scrub) {
        logScrubbingDisclaimer();
    }

    console.log(chalk.blue('\nScanning and processing files...'));
    let fileCount = 0;
    let filesToProcess = [];
    let skippedCount = 0; // Keep track of skipped files (binary/inacessible/read errors)

    // Collect file paths respecting ignores
    try {
        for await (const file of walkDir(rootDir, ig, rootDir)) { // Pass ig instance
            filesToProcess.push(file);
        }
    } catch (err) {
        console.error(chalk.red(`\nError during directory walk: ${err.message}`), err);
        if (outputStream !== process.stdout) {
            // Attempt to close the stream gracefully on error
            await new Promise((resolve) => outputStream.end(resolve));
        }
        process.exit(1);
    }

    // Sort files alphabetically by relative path
    filesToProcess.sort((a, b) => a.relativePath.localeCompare(b.relativePath));

    // Read and write content
    for (const { path: filePath, relativePath } of filesToProcess) {
        // relativePath is already POSIX format from walkDir
        const headerPath = relativePath;
        let content = null; // Initialize content to null

        try {
            content = await fs.readFile(filePath, { encoding: encoding });
            // If readFile succeeds, we assume it's processable text
            fileCount++;

            if (scrub) {
                content = scrubContent(content, relativePath); // Pass relativePath for context
            }

            // Write file block only if content was read successfully
            await writeToOutput(`--- START FILE: ${headerPath} ---`);
            await writeToOutput(content); // writeToOutput adds the newline
            await writeToOutput(`--- END FILE: ${headerPath} ---`);
            await writeToOutput(''); // Add an empty line between files (results in two newlines total)

        } catch (error) {
            // Handle file reading errors or binary detection
            skippedCount++; // Increment skipped count for any error during processing this file
            if (error.code === 'ERR_INVALID_ARG_VALUE' || (error instanceof TypeError && error.message.includes('argument must be of type string or an instance of Buffer'))) {
                // This often indicates trying to read a binary file as text
                console.warn(chalk.yellow(`   Skipping binary or non-${encoding} file: ${relativePath}`));
                // Optional: Write markers for skipped files? Could bloat output.
                // await writeToOutput(`--- START FILE: ${headerPath} ---`);
                // await writeToOutput(`[SKIPPED: BINARY OR ENCODING ERROR]`);
                // await writeToOutput(`--- END FILE: ${headerPath} ---`);
                // await writeToOutput('');
            } else if (error.code === 'EACCES' || error.code === 'EPERM') {
                console.warn(chalk.yellow(`   Skipping inaccessible file: ${relativePath}`));
            } else {
                console.warn(chalk.yellow(`   Warning: Could not read or process file ${relativePath}: ${error.message}`));
            }
            // Do not write START/END markers if the file couldn't be read/processed
        }
    }

    // --- Finalize ---
    const endMessage = `Processed ${fileCount} text files${skippedCount > 0 ? ` (skipped ${skippedCount} due to errors/encoding/permissions)` : ''}.`;
    if (outputStream !== process.stdout) {
        // Ensure buffer is flushed and file is closed
        await new Promise((resolve, reject) => {
            outputStream.end((err) => {
                if (err) return reject(err);
                console.log(chalk.green(`\n${endMessage} Output written to ${outputFilePath}`));
                resolve();
            });
        });
    } else {
        // For stdout, just log the message. stream.end() was replaced.
        console.log(chalk.green(`\n${endMessage} Output sent to stdout.`));
    }
}