// lib/tree.js
import fs from 'fs/promises';
import path from 'path';
import chalk from 'chalk';

async function generateTreeRecursive(dir, prefix, ignoreInstance, currentDepth, maxDepth, rootDir) {
    if (currentDepth > maxDepth) {
        return '';
    }

    let treeString = '';
    let entries;
    try {
        entries = await fs.readdir(dir, { withFileTypes: true });
    } catch (err) {
        if (err.code === 'EACCES' || err.code === 'EPERM') {
            // Logged in main walk, maybe silence here or make less prominent
            // console.warn(chalk.gray(`   (Tree: Skipping inaccessible directory: ${path.relative(rootDir, dir)})`));
        } else {
            console.warn(chalk.yellow(`   Warning: Could not read directory ${path.relative(rootDir, dir)} for tree generation: ${err.message}`));
        }
        return ''; // Skip directories we can't read/access
    }

    // Filter entries based on ignore rules *before* sorting and processing
    const filteredEntries = entries.filter(entry => {
        const entryPath = path.join(dir, entry.name);
        // Use POSIX separators for relative paths for consistent ignore matching
        const relativePath = path.relative(rootDir, entryPath).split(path.sep).join(path.posix.sep);
        const relativePathWithSlash = entry.isDirectory() ? relativePath + '/' : relativePath; // For directory matching

        // --- CORRECTED CHECK ---
        // Check the relative path (with potential trailing slash for dirs) and the entry name.
        // Do NOT prepend '/' here. The ignore library handles patterns starting with '/' correctly
        // against relative paths without a leading '/'.
        const isIgnored = ignoreInstance.ignores(relativePathWithSlash) || ignoreInstance.ignores(entry.name);
        // --- END CORRECTION ---

        // if (isIgnored) console.log(`Tree ignoring: ${relativePathWithSlash}`); // Debug log
        return !isIgnored;
    });


    // Sort entries: directories first, then files, alphabetically
    filteredEntries.sort((a, b) => {
        if (a.isDirectory() && !b.isDirectory()) return -1;
        if (!a.isDirectory() && b.isDirectory()) return 1;
        return a.name.localeCompare(b.name);
    });

    for (let i = 0; i < filteredEntries.length; i++) {
        const entry = filteredEntries[i];
        const isLast = i === filteredEntries.length - 1;
        const connector = isLast ? '└── ' : '├── ';
        const entryPath = path.join(dir, entry.name);

        treeString += `${prefix}${connector}${entry.name}\n`;

        if (entry.isDirectory()) {
            const newPrefix = prefix + (isLast ? '    ' : '│   ');
            // Pass rootDir down for correct relative path calculation in nested calls/errors
            treeString += await generateTreeRecursive(entryPath, newPrefix, ignoreInstance, currentDepth + 1, maxDepth, rootDir);
        }
    }
    // Remove trailing newline if treeString is not empty, otherwise return empty
    return treeString.length > 0 ? treeString.replace(/\n$/, '') : '';
}

export async function generateTree(rootDir, ignoreInstance, maxDepth) {
    console.log(chalk.blue(`Generating directory tree (max depth: ${maxDepth})...`));
    // Pass rootDir to the recursive function start
    const treeContent = await generateTreeRecursive(rootDir, '', ignoreInstance, 1, maxDepth, rootDir);
    console.log(chalk.green('Directory tree generated.'));
    // Use just basename for display root in the header
    const displayRoot = path.basename(rootDir);
    // Construct the final string, ensuring newline after the root if content exists
    return `Directory Tree (Root: ${displayRoot}, Max Depth: ${maxDepth}):\n${displayRoot}${treeContent ? '\n' + treeContent : ''}`;
}